from setuptools import setup


setup(
    name="mushroom_cli",
    version="0.2.2",
    author="Asriel",
    description=("This is a library for generating the command line interfaces for python scripts."),
    license="MIT",
    packages=['mushroom'],
    author_email="sariellian@icloud.com",
    platforms="any",
    zip_safe = False,
)